###
###

.genome <- "NCBI36"

.onLoad <- function(libname, pkgname)
{
    extdata_dirpath <- system.file("extdata", package=pkgname,
                                   lib.loc=libname, mustWork=TRUE)

    ## Make Seqinfo object.
    seqinfo_path <- file.path(extdata_dirpath, "seqinfo.txt")
    seqtable <- read.table(seqinfo_path, stringsAsFactors=FALSE)
    seqnames <- seqtable[[1L]]
    seqinfo <- Seqinfo(
        seqnames=seqnames,
        seqlengths=seqtable[[2L]],
        isCircular=seqtable[[3L]],
        genome=.genome
    )

    ## Make GenomeDescription object.
    reference_genome <- GenomeDescription(
        organism="Homo sapiens",
        species="Human",
        provider="NCBI",
        provider_version=.genome,
        release_date="Mar. 2006",
        release_name="NCBI Build 36.1",
        seqinfo=seqinfo
    )

    ## Make list of "sequence name translation tables".
    compatible_genomes <- list(
        BSgenome.Hsapiens.UCSC.hg18=
            structure(seqnames,
                      names=paste("chr", c(1:22, "X", "Y"), sep=""))
    )

    ## Make and export SNPlocs object.
    snps <- newSNPlocs(
        provider="dbSNP",
        provider_version="20090506",
        release_date="May 5, 2009",
        release_name="Build 130",
        download_url="ftp://ftp.ncbi.nih.gov/snp/organisms/human_9606/ASN1_flat/",
        download_date=NA_character_,
        reference_genome=reference_genome,
        compatible_genomes=compatible_genomes,
        data_pkgname=pkgname,
        data_dirpath=extdata_dirpath
    )

    ns <- asNamespace(pkgname)

    objname <- pkgname
    assign(objname, snps, envir=ns)
    namespaceExport(ns, objname)

    ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    ## TODO: Deprecate all the functions below in BioC 2.12
    ##

    ## Define and export getSNPcount().
    getSNPcount <- function() snpcount(snps)
    objname <- "getSNPcount"
    assign(objname, getSNPcount, envir=ns)
    namespaceExport(ns, objname)

    ## Define and export getSNPlocs().
    getSNPlocs <- function(seqname, as.GRanges=FALSE, caching=TRUE)
        snplocs(snps, seqname, as.GRanges=as.GRanges, caching=caching)
    objname <- "getSNPlocs"
    assign(objname, getSNPlocs, envir=ns)
    namespaceExport(ns, objname)

    ## Define and export rsid2loc(), rsid2alleles(), and rsidsToGRanges().
    rsid2loc <- function(rsids, caching=TRUE)
        snpid2loc(snps, rsids, caching=caching)
    objname <- "rsid2loc"
    assign(objname, rsid2loc, envir=ns)
    namespaceExport(ns, objname)

    rsid2alleles <- function(rsids, caching=TRUE)
        snpid2alleles(snps, rsids, caching=caching)
    objname <- "rsid2alleles"
    assign(objname, rsid2alleles, envir=ns)
    namespaceExport(ns, objname)

    rsidsToGRanges <- function(rsids, caching=TRUE)
        snpid2grange(snps, rsids, caching=caching)
    objname <- "rsidsToGRanges"
    assign(objname, rsidsToGRanges, envir=ns)
    namespaceExport(ns, objname)

    ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    ## SNP injection interface (see BSgenome package for how this interface
    ## is used).
    ## TODO: The SNP injection mechanism needs to be modernized and the
    ## modernized version should not use this ugly interface anymore.
    ##

    ## Define and export .loadLoc() and .loadAlleles().
    .loadLoc <- function(seqname)
        BSgenome:::.load_raw_snplocs(snps, seqname, TRUE)$loc
    objname <- ".loadLoc"
    assign(objname, .loadLoc, envir=ns)
    namespaceExport(ns, objname)

    .loadAlleles <- function(seqname)
        BSgenome:::.load_raw_snplocs(snps, seqname, TRUE)$alleles
    objname <- ".loadAlleles"
    assign(objname, .loadAlleles, envir=ns)
    namespaceExport(ns, objname)

    ## Export 'compatible_genomes' as 'COMPATIBLE_BSGENOMES'.
    objname <- "COMPATIBLE_BSGENOMES"
    assign(objname, compatible_genomes, envir=ns)
    namespaceExport(ns, objname)    
}

